/////////////////////////////////////////////////////////////////////////////
// Name:        wx/msw/prntcustom.h
// Purpose:     Support classes for creating custom print dialogs
// Author:      Hans de Ruiter
// Modified by:
// Created:     03/11/2010
// RCS-ID:      $ID: msw/prntcustom.h $
// Copyright:   (c) Hans de Ruiter
// Licence:     wxWindows Licence
/////////////////////////////////////////////////////////////////////////////

#ifndef _WX_PRINTCUSTOM_H_MSW_
#define _WX_PRINTCUSTOM_H_MSW_

#include "wx/printcustom.h"

#if wxUSE_PRINTING_ARCHITECTURE && (!defined(__WXUNIVERSAL__) || !wxUSE_POSTSCRIPT_ARCHITECTURE_IN_MSW)
// ---------------------------------------------------------------------------
// wxWindowsNativePrinterInfo: Native MS Windows printer information for 
// wxPrinterInfo
// ---------------------------------------------------------------------------

class wxWindowsNativePrinterInfo : public wxNativePrinterInfo
{
public:
	/** Creates a wxWindowsNativePrinterInfo object for the given printer.
	 *
	 * @param printerName the printer's name
	 * @param isDefault set to true if this is the default printer, and
	 * false otherwise
	 */
	wxWindowsNativePrinterInfo(const wxString &printerName, bool isDefault);

	virtual wxSharedPtr<wxPrintPaperDatabase> GetSupportedPaperSizes() const;

	virtual void getNonPrintableMargins(
		unsigned int *leftMargin, unsigned int *rightMargin,
		unsigned int *topMargin, unsigned int *bottomMargin) const;
	
	virtual bool IsSamePrinter(const wxPrintData &printData) const;

    virtual const wxPrintData GetPrintData();

	virtual void UpdatePrintData(wxPrintData &printData) const;

	virtual bool SetPrintDataPaperType(wxPaperSize paperID, wxPrintData &printData) const;

private:
	wxString printerName;
};

// ---------------------------------------------------------------------------
// wxWindowsPrinterPrefsDialog: Encapsulates the MS Windows printer 
// preferences dialog
// ---------------------------------------------------------------------------

class wxWindowsPrinterPrefsDialog : public wxNativePrinterPrefsDialog
{
public:
    wxWindowsPrinterPrefsDialog(wxWindow *parent, wxPrintData *data = NULL);

    virtual ~wxWindowsPrinterPrefsDialog();

    virtual int ShowModal();

    virtual wxPrintData& GetPrintData();

private:
	/** The print data object.
	 */
	wxPrintData m_printData;

	/** The printer's paper database
	 */
	wxSharedPtr<wxPrintPaperDatabase> m_paperDatabase;

	/** Pointer to the parent window.
	 */
	wxWindow *m_dialogParent;
};

#endif

#endif
    // _WX_PRINTCUSTOM_H_BASE_