/////////////////////////////////////////////////////////////////////////////
// Name:        wx/printcustom.cpp
// Purpose:     Support classes for creating custom print dialogs
// Author:      Hans de Ruiter
// Modified by:
// Created:     03/11/2010
// RCS-ID:      $ID: printcustom.cpp $
// Copyright:   (c) Hans de Ruiter
// Licence:     wxWindows Licence
/////////////////////////////////////////////////////////////////////////////

// ===========================================================================
// declarations
// ===========================================================================

// ---------------------------------------------------------------------------
// headers
// ---------------------------------------------------------------------------

// For compilers that support precompilation, includes "wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif
 
#include "wx/printcustom.h"

#if wxUSE_PRINTING_ARCHITECTURE

// ===========================================================================
// implementation
// ===========================================================================

// ---------------------------------------------------------------------------
// Printer System
// ---------------------------------------------------------------------------

wxNativePrinterInfo::wxNativePrinterInfo(bool isDefault)
{
    m_isDefault = isDefault;
}

wxNativePrinterInfo::~wxNativePrinterInfo()
{
    // Nothing to do here (but need a virtual destructor or we'll get 
	// memory leaks
}

bool wxNativePrinterInfo::IsDefaultPrinter() const
{
    return m_isDefault;
}

wxPrinterInfo::wxPrinterInfo(
    wxSharedPtr<wxNativePrinterInfo> nativePrinterInfo) :
        m_nativePrinterInfo(nativePrinterInfo)
{ 
    if(!m_nativePrinterInfo)
    {
		throw "NULL pointer error";
    }
}
    
wxPrinterInfo::~wxPrinterInfo()
{
	// Nothing to do here
}

wxSharedPtr<wxPrintPaperDatabase> wxPrinterInfo::GetSupportedPaperSizes()
{
	if(!m_paperDatabase)
	{
		m_paperDatabase = m_nativePrinterInfo->GetSupportedPaperSizes();
	}

	return m_paperDatabase;
}

void wxPrinterInfo::getNonPrintableMargins(
	unsigned int *leftMargin, unsigned int *rightMargin,
		unsigned int *topMargin, unsigned int *bottomMargin)
{
	m_nativePrinterInfo->getNonPrintableMargins(
		leftMargin, rightMargin, topMargin, bottomMargin);
}

bool wxPrinterInfo::IsSamePrinter(const wxPrintData &printData) const
{
    return m_nativePrinterInfo->IsSamePrinter(printData);
}

wxPrintData wxPrinterInfo::GetPrintData()
{
    return m_nativePrinterInfo->GetPrintData();
}

void wxPrinterInfo::UpdatePrintData(wxPrintData &printData) const
{
    m_nativePrinterInfo->UpdatePrintData(printData);
}

bool wxPrinterInfo::SetPrintDataPaperType(wxPaperSize paperID, wxPrintData &printData) const
{
	return m_nativePrinterInfo->SetPrintDataPaperType(paperID, printData);
}

bool wxPrinterInfo::IsDefaultPrinter() const
{
	return m_nativePrinterInfo->IsDefaultPrinter();
}

wxSharedPtr<wxPrinterInfo> wxPrinterSysInfo::GetDefaultPrinter()
{
	wxVector<wxSharedPtr<wxPrinterInfo>> printers = wxPrinterSysInfo::GetAvailablePrinters();

	for(wxVector<wxSharedPtr<wxPrinterInfo>>::iterator currPrinter = printers.begin();
		currPrinter != printers.end(); ++currPrinter)
	{
		if((*currPrinter)->IsDefaultPrinter())
		{
			return *currPrinter;
		}
	}

	return wxSharedPtr<wxPrinterInfo>();
}

wxSharedPtr<wxPrinterInfo> wxPrinterSysInfo::GetPrinterByPrintData(const wxPrintData* printData)
{
	wxVector<wxSharedPtr<wxPrinterInfo>> printers = wxPrinterSysInfo::GetAvailablePrinters();

	for(wxVector<wxSharedPtr<wxPrinterInfo>>::iterator currPrinter = printers.begin();
		currPrinter != printers.end(); ++currPrinter)
	{
		if((*currPrinter)->IsSamePrinter(*printData))
		{
			return *currPrinter;
		}
	}

	return wxSharedPtr<wxPrinterInfo>();
}

wxVector<wxSharedPtr<wxPrinterInfo>> wxPrinterSysInfo::availablePrinters;

#if defined(__WXMSW__) && !defined(__WXUNIVERSAL__)
#include "wx/msw/prntcustom.h"
#define wxOSNativePrinterPrefsDialog wxWindowsPrinterPrefsDialog
#else
#error "Printcustom not implemented for your OS"
#endif

wxPrinterPrefsDialog::wxPrinterPrefsDialog(wxWindow *parent, wxPrintData *data) :
            m_nativeDialog(new wxOSNativePrinterPrefsDialog(parent, data))
{
    if(!m_nativeDialog)
    {
		throw "Failed to create native printer prefs dialog";
    }
}

wxPrinterPrefsDialog::~wxPrinterPrefsDialog()
{
    // Nothing to do here
}

int wxPrinterPrefsDialog::ShowModal()
{
	return m_nativeDialog->ShowModal();
}

wxPrintData& wxPrinterPrefsDialog::GetPrintData()
{
	return m_nativeDialog->GetPrintData();
}


#endif