/////////////////////////////////////////////////////////////////////////////
// Name:        wxPrintCustomDemo.cpp
// Purpose:     Demo app for using wxPrintCustom to get details about 
//              printers
// Author:      Hans de Ruiter
// Modified by:
// Created:     13/10/2011
// RCS-ID:      $ID: wxPrintCustomTest.cpp $
// Copyright:   (c) Hans de Ruiter
// Licence:     wxWindows Licence
/////////////////////////////////////////////////////////////////////////////

// ===========================================================================
// declarations
// ===========================================================================

// ---------------------------------------------------------------------------
// headers
// ---------------------------------------------------------------------------

// For compilers that support precompilation, includes "wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif
 
#include "wxPrintCustomDemo.h"


// ===========================================================================
// implementation
// ===========================================================================

// ----------------------------------------------------------------------------
// MyApp
// ----------------------------------------------------------------------------

IMPLEMENT_APP(MyApp)

bool MyApp::OnInit()
{
    if(!wxApp::OnInit())
    {
        return false;
    }

    wxInitAllImageHandlers();

	// Create the main frame window
    // ----------------------------

    MyFrame* frame = new MyFrame((wxFrame *) NULL, wxT("wxWidgets Custom Print Demo"),
                                 wxPoint(0, 0), wxSize(400, 400));

    frame->Centre(wxBOTH);
    frame->Show(true);

    SetTopWindow(frame);

    return true;
}

BEGIN_EVENT_TABLE(MyFrame, wxFrame)
    EVT_MENU(wxID_EXIT, MyFrame::OnExit)
 //   EVT_MENU(WXPRINT_PAGE_SETUP, MyFrame::OnPageSetup)
    EVT_MENU(wxID_ABOUT, MyFrame::OnPrintAbout)
END_EVENT_TABLE()

MyFrame::MyFrame(wxFrame *frame, const wxString &title, const wxPoint &pos, const wxSize &size)
        : wxFrame(frame, wxID_ANY, title, pos, size)
{
	// Get a list of printers
	availablePrinters = wxPrinterSysInfo::GetAvailablePrinters();
	supportedPaperSizes.resize(availablePrinters.size());


	// Make a menubar
    wxMenu *file_menu = new wxMenu;
    file_menu->Append(wxID_EXIT, wxT("E&xit"), wxT("Exit program"));

    wxMenu *help_menu = new wxMenu;
    help_menu->Append(wxID_ABOUT, wxT("&About"), wxT("About this demo"));

    wxMenuBar *menu_bar = new wxMenuBar;

    menu_bar->Append(file_menu, wxT("&File"));
    menu_bar->Append(help_menu, wxT("&Help"));

    // Associate the menu bar with the frame
    SetMenuBar(menu_bar);


	// ----- Create the main GUI ------

	wxBoxSizer* topSizer = new wxBoxSizer(wxHORIZONTAL);
	wxPanel *basePanel = new wxPanel(this);
	topSizer->Add(basePanel, 1, wxEXPAND | wxALL, 0);
	SetSizer(topSizer);

	wxBoxSizer *vertSizer = new wxBoxSizer(wxVERTICAL);
    basePanel->SetSizer(vertSizer);

	// The printer selection GUI
	wxStaticText *printerSettingsLabel = 
		new wxStaticText(basePanel, wxID_ANY, _("Printer:"), 
		wxDefaultPosition, wxDefaultSize, 0);
	printerSettingsLabel->SetFont(printerSettingsLabel->GetFont().Bold());
    vertSizer->Add(printerSettingsLabel, 0, wxALIGN_LEFT | wxALL, 5);
    wxBoxSizer *printerSettingsSizer  = new wxBoxSizer(wxHORIZONTAL);
    printerChoice = new wxChoice(basePanel, wxID_ANY);
    int selPrinter = 0;
    for(unsigned int i = 0; i < availablePrinters.size(); ++i)
    {
        printerChoice->AppendString(
                 availablePrinters[i]->GetPrintData().GetPrinterName());
        if(availablePrinters[i]->IsDefaultPrinter())
        {
			// Choose the system default printer as the app's default printer
            selPrinter = i;
        }
    }
	printerSettingsSizer->Add(printerChoice, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
	wxButton *printerPrefsButton = new wxButton(basePanel, wxID_ANY, _("Preferences"));
	printerSettingsSizer->Add(printerPrefsButton, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
	vertSizer->Add(printerSettingsSizer, 0, wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT | wxBOTTOM, 5);
	
	vertSizer->AddSpacer(5);

	// The paper selection GUI
	// Paper setup controls
	wxStaticText *paperSettingsLabel = 
		new wxStaticText(basePanel, wxID_ANY, _("Paper:"), 
		wxDefaultPosition, wxDefaultSize, 0);
	paperSettingsLabel->SetFont(paperSettingsLabel->GetFont().Bold());
    vertSizer->Add(paperSettingsLabel, 0, wxALIGN_LEFT | wxALL, 5);
	wxBoxSizer *paperSettingsSizer  = new wxBoxSizer(wxHORIZONTAL);
	paperChoice = new wxChoice(basePanel, wxID_ANY);
	paperSettingsSizer->Add(paperChoice, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
	wxBoxSizer *orientationSettingsSizer  = new wxBoxSizer(wxVERTICAL);
	paperPortraitRadioButton = new wxRadioButton(basePanel, wxID_ANY, _("&Portrait"), wxDefaultPosition, wxDefaultSize, wxRB_GROUP);
	orientationSettingsSizer->Add(paperPortraitRadioButton, 0, wxALIGN_LEFT | wxALL, 1);
    paperLandscapeRadioButton = new wxRadioButton(basePanel, wxID_ANY, _("&Landscape"), wxDefaultPosition, wxDefaultSize);
	orientationSettingsSizer->Add(paperLandscapeRadioButton, 0, wxALIGN_LEFT | wxALL, 1);
    paperSettingsSizer->Add(orientationSettingsSizer, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
	vertSizer->Add(paperSettingsSizer, 0, wxGROW | wxLEFT | wxRIGHT | wxBOTTOM, 5);

	// Connect up events
	Connect(printerChoice->GetId(), wxEVT_COMMAND_CHOICE_SELECTED,
		wxCommandEventHandler(MyFrame::OnPrinterSelected), NULL, this);
	Connect(printerPrefsButton->GetId(), wxEVT_COMMAND_BUTTON_CLICKED, 
		wxCommandEventHandler(MyFrame::OnPrinterPrefsClicked), NULL, this);
	Connect(paperChoice->GetId(), wxEVT_COMMAND_CHOICE_SELECTED,
		wxCommandEventHandler(MyFrame::OnPaperSelected), NULL, this);
	Connect(paperPortraitRadioButton->GetId(), wxEVT_COMMAND_RADIOBUTTON_SELECTED,
		wxCommandEventHandler(MyFrame::OnPaperOrientationSelected), NULL, this);
	Connect(paperLandscapeRadioButton->GetId(), wxEVT_COMMAND_RADIOBUTTON_SELECTED,
		wxCommandEventHandler(MyFrame::OnPaperOrientationSelected), NULL, this);

	// Initialise the settings
	setPrinter(selPrinter);
}

void MyFrame::OnExit(wxCommandEvent& WXUNUSED(event))
{
	Close(true);
}

void MyFrame::OnPrintAbout(wxCommandEvent& WXUNUSED(event))
{
	(void)wxMessageBox(wxT("wxWidgets custom print dialog toolkit demo\nAuthor: Hans de Ruiter"),
                       wxT("About custom print dialog toolkit demo"), wxOK|wxCENTRE);
}

void MyFrame::OnPrinterSelected(wxCommandEvent& event)
{
	int index = printerChoice->GetSelection();
	setPrinter(index);
}

void MyFrame::OnPrinterPrefsClicked(wxCommandEvent& event)
{
	wxPrinterPrefsDialog prefsDialog(this, &(data.GetPrintData()));

	if(prefsDialog.ShowModal() == wxID_OK)
	{
		data.SetPrintData(prefsDialog.GetPrintData());
		
		updateGUI();
	}
}

void MyFrame::OnPaperSelected(wxCommandEvent& event)
{
	if(currPrinterIndex < supportedPaperSizes.size() && supportedPaperSizes[currPrinterIndex])
	{
		int index = paperChoice->GetSelection();
		if(index < (int)supportedPaperSizes[currPrinterIndex]->GetCount())
		{
			wxPrintPaperType *paperType = supportedPaperSizes[currPrinterIndex]->Item(index);

			wxPaperSize paperID = paperType->GetId();
			wxPrintData &printData = data.GetPrintData();
			if(!availablePrinters[currPrinterIndex]->SetPrintDataPaperType(paperID, printData))
			{
				// Backup method
				printData.SetPaperId(paperID);
				wxSize paperDims = paperType->GetSizeMM();
				printData.SetPaperSize(paperDims);
				printData.ConvertToNative();
			}

			updateGUI();
		}
	}
}

void MyFrame::OnPaperOrientationSelected(wxCommandEvent& event)
{
	wxPrintData &printData = data.GetPrintData();

	if(paperPortraitRadioButton->GetId() == event.GetId())
	{
		printData.SetOrientation(wxPORTRAIT);
	}
	else
	{
		printData.SetOrientation(wxLANDSCAPE);
	}
}

void MyFrame::setPrinter(unsigned int index)
{
	if(index < availablePrinters.size())
	{
		if(currPrinterIndex != index)
		{
			currPrinterIndex = index;
			printerChoice->SetSelection(index);
			availablePrinters[index]->UpdatePrintData(data.GetPrintData());
			if(!supportedPaperSizes[index])
			{
				supportedPaperSizes[index] = 
					availablePrinters[index]->GetSupportedPaperSizes();
			}

			updatePaperSelectionGUI();

			updateGUI();
		}
	}
	else
	{
		throw "Index out of bounds";
	}
}

void MyFrame::updatePaperSelectionGUI()
{
	if(currPrinterIndex < supportedPaperSizes.size())
	{
		paperChoice->Clear();

		for(size_t i = 0; i < supportedPaperSizes[currPrinterIndex]->GetCount(); ++i)
		{
			paperChoice->AppendString(
				supportedPaperSizes[currPrinterIndex]->Item(i)->GetName());
		}
	}
}

void MyFrame::updateGUI()
{
	if(currPrinterIndex < supportedPaperSizes.size() && supportedPaperSizes[currPrinterIndex])
	{
		// Set the paper
		wxPrintData &printData = data.GetPrintData();
		wxPaperSize paperSize = printData.GetPaperId();

		wxPrintPaperType *paperType = NULL;
		if(paperSize != wxPAPER_NONE)
		{
			paperType = supportedPaperSizes[currPrinterIndex]->FindPaperType(paperSize);
		}
		if(!paperType)
		{
			// Try to get the paper type by size
			wxSize paperDims = printData.GetPaperSize();
			paperDims *= 10; // Need in tenths of a millimetre

			paperType = supportedPaperSizes[currPrinterIndex]->FindPaperType(paperDims);
		}

		if(paperType)
		{
			size_t paperIndex;
			size_t numPapers = supportedPaperSizes[currPrinterIndex]->GetCount();
			for(paperIndex = 0; paperIndex < numPapers; ++paperIndex)
			{
				if(paperType == supportedPaperSizes[currPrinterIndex]->Item(paperIndex))
				{
					break;
				}
			}

			if(paperIndex < numPapers)
			{
				paperChoice->SetSelection(paperIndex);
			}
		}

		// Set the paper orientation
		if(printData.GetOrientation() == wxLANDSCAPE)
		{
			paperLandscapeRadioButton->SetValue(true);
		}
		else
		{
			paperPortraitRadioButton->SetValue(true);
		}
	}
}