/////////////////////////////////////////////////////////////////////////////
// Name:        wxPrintCustomDemo.h
// Purpose:     Demo app for using wxPrintCustom to get details about 
//              printers
// Author:      Hans de Ruiter
// Modified by:
// Created:     13/10/2011
// RCS-ID:      $ID: wxPrintCustomTest.cpp $
// Copyright:   (c) Hans de Ruiter
// Licence:     wxWindows Licence
/////////////////////////////////////////////////////////////////////////////

// ===========================================================================
// declarations
// ===========================================================================

#ifndef __WXPRINTCUSTOMTEST_H__
#define __WXPRINTCUSTOMTEST_H__

// ---------------------------------------------------------------------------
// headers
// ---------------------------------------------------------------------------

// For compilers that support precompilation, includes "wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif
 
#include "wx/printcustom.h"


// Application class
class MyApp: public wxApp
{
public:
    MyApp() {}

    virtual bool OnInit();
};

DECLARE_APP(MyApp)



// Main window
class MyFrame: public wxFrame
{
public:
    MyFrame(wxFrame *frame, const wxString& title, const wxPoint& pos, const wxSize& size);

// ##### FIXME! #####

    void OnExit(wxCommandEvent& event);

    void OnPrintAbout(wxCommandEvent& event);

	void OnPrinterSelected(wxCommandEvent& event);

	void OnPrinterPrefsClicked(wxCommandEvent& event);

	void OnPaperSelected(wxCommandEvent& event);

	void OnPaperOrientationSelected(wxCommandEvent& event);

private:
	/** Sets the current printer by index.
	 */
	void setPrinter(unsigned int index);

	/** Updates the paper type choice control to match the new printer.
	 */
	void updatePaperSelectionGUI();

	/** Sets the GUI to match the current printer settings (i.e., the 
	 * settings in this class' data field).
	 *
	 * NOTE: This does NOT include updating the selected printer. Any
	 * changes to the selected printer should be handled by a call to
	 * setPrinter(), which will also call this method.
	 *
	 * This will also refresh the preview display.
	 */
	void updateGUI();

	/** The list of available printers.
	 */
	wxVector<wxSharedPtr<wxPrinterInfo>> availablePrinters;

	/** The supported paper sizes for each printer.
	 */
	wxVector<wxSharedPtr<wxPrintPaperDatabase>> supportedPaperSizes;

	/** The printer choice control.
	 */
	wxChoice *printerChoice;

	/** The paper choice control.
	 */
	wxChoice *paperChoice;

	/** The paper portrait orientation control.
	 */
	wxRadioButton *paperPortraitRadioButton;

	/** The paper portrait orientation control.
	 */
	wxRadioButton *paperLandscapeRadioButton;

	/** Stores the printer settings.
	 */
	wxPrintDialogData data;

	/** Index of the currently selected printer.
	 */
	unsigned int currPrinterIndex;

    DECLARE_EVENT_TABLE()
};

#endif
